/*
 
 Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of SAMstat.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */

#include <ctype.h>

#include "interface.h"
#include "nuc_code.h"
#include "misc.h"
#include "io.h"
#include "rRNAfilter.h"




struct fasta* get_fasta(struct fasta* p,char *infile,int solid,int server)
{
	p = (struct fasta*) malloc(sizeof(struct fasta));
	assert(p != 0);
	p->string = 0;
	p->string =  get_input_into_string(p->string,infile,server);
	if(!p->string){
		fprintf(stderr,"Analysing input %s ... nothing\n",infile);
		exit(-1);
	}
	
	p = read_fasta(p);
	
	return p;
}


unsigned char* get_input_into_string(unsigned char* string,char* infile,int server)
{
	int i = 0;   
	int string_length = 2;
	char c = 0;    
	
	/*
	 // added these items Jody Hey 11/29/07
	 int maxl, tempmaxl, sl;
	 char* cc;
	 int j;
	 */
	FILE *file = 0;
	if(infile){
		if (!(file = fopen( infile, "r" ))){
			return 0;
 			fprintf(stderr,"Cannot open file '%s'\n", infile);
			exit(-1);
		}
		if (fseek(file,0,SEEK_END) != 0){
			(void)fprintf(stderr, "ERROR: fseek failed\n");
			(void)exit(EXIT_FAILURE);
		}
		i= ftell (file);
		if (fseek(file,0,SEEK_START) != 0){
			(void)fprintf(stderr, "ERROR: fseek failed\n");
			(void)exit(EXIT_FAILURE);
		}
		if(!string){
			string = malloc ((i+1+18)* sizeof(unsigned char));
			assert(string != 0);
		}
		fread(string,sizeof(unsigned char), i, file);
		string[i] = 0;
		fclose(file);
		/*
		 // added this section of code, Jody Hey 11/29/07
		 // gets the longest line length,  so that we can use fgets later, then uses fgets to get a string of the file contents
		 maxl = tempmaxl = 0;
		 sl = strlen(string) - 1;
		 for  (j = 0; j <= sl; j++)
		 if (string[j]=='\n' || j == sl){
		 if (maxl < (j - tempmaxl))
		 maxl = tempmaxl;
		 tempmaxl = j;
		 }
		 maxl++;
		 string[0] = 0;
		 if (!(file = fopen( infile, "r" ))){
		 fprintf(stderr,"Cannot open file '%s'\n", infile);
		 exit(1);
		 }
		 j = 0;
		 cc = string;
		 while (fgets(cc, maxl,file))
		 cc = &string[strlen(string)]; 
		 fclose(file);
		 // end of added section */
	}else{
		//fprintf(stderr,"from STDIN\n");
		if (!isatty(0) && server == 0 ){
			string = malloc(sizeof(unsigned  char*)*string_length);
			assert(string != 0);
			while (!feof (stdin)){
				c = getc(stdin);
				if (i == string_length){
					string_length <<=1;
					string = realloc(string,sizeof(unsigned char)*(string_length+18));
				}
				string[i] = c;
				i++;
			}
			string[i-1] = 0;
			//fprintf(stderr,"STRING:%s\n",string);
		}else{
			return 0;
		}
	}
	return string;
}


struct fasta* read_fasta(struct fasta* aln)
{
	int c = 0;
	int n = 0;
	int i = 0;
	int j = 0;
	int len = 0;
	int stop = 0;
	int nbytes;
	
	nbytes = strlen((char*) aln->string);
	aln->numseq = 0;
	aln->max_len = -1;
	//aln->org_seq = 0;
	stop = 0;
	
	//count filenames....
	for (i =0;i < nbytes;i++){
		if (aln->string[i] == '>'&& stop == 0){
			aln->numseq++;
			stop = 1;
		}else if (aln->string[i] == '\n'){
			stop = 0;
		}else if(aln->string[i] == '\r'){
			aln->string[i] = '\n';
		}
	}
	
	aln->suffix = 0;
	aln->sn = malloc(sizeof(unsigned char*)*aln->numseq);
	assert(aln->sn  !=0);
	//aln->c = 0;
	
	aln->s_index = malloc(sizeof(int)*(aln->numseq+1));
	assert(aln->s_index != 0);
	for (i =0;i < nbytes;i++){
		if (aln->string[i] == '>'){
			if(aln->max_len < len){
				aln->max_len = len;
			}
			
			len = 0;
			j = i+1;
			while(aln->string[j] != '\n'){
				//	fprintf(stderr,"%c",aln->string[j]);
				j++;
			}
			//fprintf(stderr,"	%d\n",j-i);
			aln->sn[c] = malloc(sizeof(char)*(j-i));
			assert(aln->sn[c] != 0);
			aln->s_index[c] = n;
			j = i+1;
			len = 0;
			while(aln->string[j] != '\n'){
				if(isspace((int)aln->string[j])){
					aln->sn[c][len] = '_';
				}else{ 
					aln->sn[c][len] = aln->string[j];
				}
				len++;
				j++;
			}
			aln->sn[c][len] = 0;
			aln->string[n] = 'X';
			n++;
			i+= j-i;
			c++;
		}else if(isalnum((int)aln->string[i])){// if (aln->string[i] != '\n' && aln->string[i] != 0 ){
			aln->string[n] =   nuc_code5[(int)aln->string[i]];//  toupper(aln->string[i]);
			n++;
			len++;
		}
		//}
	}
	
	
	aln->s_index[c] = n;
	aln->string[n] = 'X';
	aln->string[n+1] = 0;
	
	aln->string_len = n+1;
	return aln;
}

void print_sam(struct read_info* ri,FILE* out,int good)
{
	char pline[MAX_LINE];
	char pseq[MAX_LINE];
	char alphabet[] = "ACGTN";
	int i,c;
	
	if(!good){
		ri->flag[0] |=  0x200;
	}
	
	for(i =0;i < ri->len;i++){
		pseq[i] = alphabet[ri->seq[i]];
	}
	pseq[ri->len] = 0;
	//fprintf(out,"\n");
	
	if(ri->remain){
	
	sprintf(pline,"%s\t%d\t%s\t%s\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s",
			ri->name ,//1 QNAME Query NAME of the read or the read pair 
			ri->flag[0],//2 FLAG bitwise FLAG (pairing, strand, mate strand, etc.) 
			ri->RNAME,//3 RNAME Reference sequence NAME 
			ri->POS,//4 POS 1-based leftmost POSition of clipped alignment 
			(int)ri->mapq,//5 MAPQ MAPping Quality (Phred-scaled) 
			ri->cigar,//6 CIGAR extended CIGAR string (operations: MIDNSHP) 
			ri->MRNM ,//7 MRNM Mate Reference NaMe (‘=’ if same as RNAME) 
			ri->MPOS ,//8 MPOS 1-based leftmost Mate POSition 
			ri->ISIZE,//9 ISIZE inferred Insert SIZE 
			pseq,//10 SEQ query SEQuence on the same strand as the reference 
			ri->qual,//11 QUAL query QUALity (ASCII-33=Phred base quality) 
			ri->remain
			);
	}else{
		sprintf(pline,"%s\t%d\t%s\t%s\t%d\t%s\t%s\t%s\t%s\t%s\t%s",
				ri->name ,//1 QNAME Query NAME of the read or the read pair 
				ri->flag[0],//2 FLAG bitwise FLAG (pairing, strand, mate strand, etc.) 
				ri->RNAME,//3 RNAME Reference sequence NAME 
				ri->POS,//4 POS 1-based leftmost POSition of clipped alignment 
				(int)ri->mapq,//5 MAPQ MAPping Quality (Phred-scaled) 
				ri->cigar,//6 CIGAR extended CIGAR string (operations: MIDNSHP) 
				ri->MRNM ,//7 MRNM Mate Reference NaMe (‘=’ if same as RNAME) 
				ri->MPOS ,//8 MPOS 1-based leftmost Mate POSition 
				ri->ISIZE,//9 ISIZE inferred Insert SIZE 
				pseq,//10 SEQ query SEQuence on the same strand as the reference 
				ri->qual//11 QUAL query QUALity (ASCII-33=Phred base quality) 
				
				);
		
	}
	c = strlen(pline);
	pline[c] = 0;
	fprintf(out, "%s\n",pline);

}

void print_seq(struct read_info* ri,FILE* out)
{
	char alphabet[] = "ACGTN";
	int i;
	if(ri->qual == 0){
		fprintf(out,">%s\n",ri->name);
		for(i =0;i < ri->len;i++){
			fprintf(out,"%c",alphabet[ri->seq[i]]);
		}
		fprintf(out,"\n");
	}else if(ri->qual[0] == '*'){
		fprintf(out,">%s\n",ri->name);
		for(i =0;i < ri->len;i++){
			fprintf(out,"%c",alphabet[ri->seq[i]]);
		}
		fprintf(out,"\n");
	}else{
		fprintf(out,"@%s\n",ri->name);
		for(i =0;i < ri->len;i++){
			fprintf(out,"%c",alphabet[ri->seq[i]]);
		}
		fprintf(out,"\n+\n");
		if(ri->qual){
			fprintf(out,"%s",ri->qual);
		}else{
			for(i =0;i < ri->len;i++){
				fprintf(out,".");
			}
		}
		fprintf(out,"\n");
	}
}


int read_sam_chunk(struct read_info** ri,struct parameters* param,FILE* file)
{
	char line[MAX_LINE];
	int column = 0; 
	int i,j,g,tmp;
	unsigned int pos;
	int read = 0;
	int c = 0;
	int hit = 0;
	
	for(i = 0; i < param->num_query;i++){
		if(ri[i]->seq){
			free(ri[i]->seq);
		}
		if(ri[i]->name){
			free(ri[i]->name);
		}
		if(ri[i]->qual){
			free(ri[i]->qual);
		}
		if(ri[i]->priors){
			free(ri[i]->priors);
		}
		
		if(ri[i]->cigar){
			free(ri[i]->cigar);
		}
		if(ri[i]->md){
			free(ri[i]->md);
		}
		
		if(ri[i]->RNAME){
			free(ri[i]->RNAME);
		}
		if(ri[i]->POS){
			free(ri[i]->POS);
		}
		if(ri[i]->MRNM){
			free(ri[i]->MRNM);
		}
		if(ri[i]->MPOS){
			free(ri[i]->MPOS);
		}
		if(ri[i]->ISIZE){
			free(ri[i]->ISIZE);
		}
		if(ri[i]->remain){
			free(ri[i]->remain);
		}
		//free(ri[i]->strand);
		
		ri[i]->RNAME = 0;
		ri[i]->POS = 0;
		ri[i]->MRNM = 0;
		ri[i]->MPOS = 0;
		ri[i]->ISIZE = 0;
		ri[i]->remain = 0;
		
				 
		ri[i]->priors = 0;
		ri[i]->seq = 0;
		ri[i]->name = 0;
		ri[i]->qual = 0;
		ri[i]->len = 0;
		ri[i]->errors = 0;
		ri[i]->cigar = 0;
		ri[i]->md = 0;
		ri[i]->a = 0;
		//ri[i]->strand = 0;
	}
	//fprintf(stderr,"READING....\n");
	while(fgets(line, MAX_LINE, file)){
		if(line[0] != '@'){
			column = 1; //<QNAME> 
			tmp = 0;
			hit = 0;
			pos = 0xFFFFFFFFu;
			for(j = 0;j < MAX_LINE;j++){
				tmp++;
				if(isspace((int)line[j])){
					break;
				}
			}
			
			ri[c]->name = malloc(sizeof(unsigned char)* tmp);
			for(j = 0;j < MAX_LINE;j++){
				
				if(isspace((int)line[j])){
					ri[c]->name[j] = 0;
					break;
				}
				ri[c]->name[j] = line[j];
			}
			
			for(i = 0; i < MAX_LINE;i++){
				if(line[i] == '\n'){
					break;
				}
				if(isspace((int)line[i])){
					column++;
					switch(column){
						case 2: // <FLAG>
							ri[c]->flag[0] = atoi(line+i+1);
							break;
						case 3: // <RNAME> 
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->RNAME = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->RNAME[g] = 0;
									break;
								}
								ri[c]->RNAME[g] = line[j];
								g++;
							}
							break;
							
						case 4: // <POS>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->POS = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->POS[g] = 0;
									break;
								}
								ri[c]->POS[g] = line[j];
								g++;
							}
							break;
						case 5: //  <MAPQ>
							
							ri[c]->mapq =  atof(line +i +1); 
							
							break;
						case 6: //  <CIGAR>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->cigar = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->cigar[g] = 0;
									break;
								}
								ri[c]->cigar[g] = line[j];
								g++;
							}
							break;
						case 7: //  <MRNM>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->MRNM = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->MRNM[g] = 0;
									break;
								}
								ri[c]->MRNM[g] = line[j];
								g++;
							}
							break;
						case 8: //  <MPOS>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->MPOS = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->MPOS[g] = 0;
									break;
								}
								ri[c]->MPOS[g] = line[j];
								g++;
							}
							break;
						case 9: //  <ISIZE>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->ISIZE = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(isspace((int)line[j])){
									ri[c]->ISIZE[g] = 0;
									break;
								}
								ri[c]->ISIZE[g] = line[j];
								g++;
							}
							break;
						case 10: // <SEQ>
							
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							
							ri[c]->seq = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								
								if(isspace((int)line[j])){
									ri[c]->seq[g] = 0;
									break;
								}
								ri[c]->seq[g] = nuc_code5[(int)line[j]];
								g++;
							}
							
							ri[c]->len = g;
							break;
						case 11: // <QUAL>
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
								if(isspace((int)line[j])){
									break;
								}
							}
							g= 0;
							ri[c]->qual = malloc(sizeof(unsigned char)* tmp);
							for(j = i+1;j < MAX_LINE;j++){
								
								if(isspace((int)line[j])){
									ri[c]->qual[g] = 0;
									break;
								}
								ri[c]->qual[g] = line[j];
								g++;
							}
							break;
						default: 
							tmp = 0;
							for(j = i+1;j < MAX_LINE;j++){
								tmp++;
							//	fprintf(stderr,"%c",line[j]);
								if(line[j] == '\n'){
									break;
								}
							}
							//fprintf(stderr,"\n");
							ri[c]->remain = malloc(sizeof(unsigned char)* tmp);
							g = 0;
							for(j = i+1;j < MAX_LINE;j++){
								if(line[j] == '\n'){
									ri[c]->remain[g] = 0;
									break;
								}
								ri[c]->remain[g] = line[j];
								g++;
							}
							
							i = MAX_LINE;
							break;
					}
				}
			}
			/*tmp = byg_end("NM:i:", line  );
			if(tmp){
				ri[c]->errors = atoi(line+tmp);
				//if(ri[c]->errors > 20){
				///fprintf(stderr,"%s\n,%c,%c,%c,%d\n",line, *(line +tmp), *(line +tmp+1),*(line +tmp+2), ri[c]->errors); 
				//}
				
			}else{
				ri[c]->errors = -1;								
			}	
			
			tmp = byg_end("MD:Z:", line  );
			if(tmp){
				g = 0;
				for(j = tmp ;j < MAX_LINE;j++){
					g++;
					if(isspace((int)line[j])){
						break;
					}
					
				}
				ri[c]->md = malloc(sizeof(unsigned char)* g);
				g = 0;
				for(j = tmp ;j < MAX_LINE;j++){
					
					if(isspace((int)line[j])){
						ri[c]->md[g] = 0;
						break;
					}
					ri[c]->md[g] = line[j];
					g++;
				}
			}
			*/
			ri[c]->hits[hit] = 0xFFFFFFFFu;
			ri[c]->identity[hit] = -2.0f;
			
			
			//fprintf(stderr,"%s\n",line);
			//fprintf(stderr,"%f\n",ri[c]->mapq);
			c++;
			read++;
			if(c == param->num_query){
				return c;
			}
		}
	}
	return c;
}



int read_fasta_fastq(struct read_info** ri,struct parameters* param,FILE *file) 
{
	int park_pos = -1;
	char line[MAX_LINE];
	int i;//,j;
	int seq_p = 0;
	int set = 0;
	int len = 0;
	int size = 0;
	int fastq = 0;
	
	//fprintf(stderr,"cleaning....\n");
	for(i = 0; i < param->num_query;i++){
		free(ri[i]->seq);
		free(ri[i]->name);
		free(ri[i]->qual);
		if(ri[i]->priors){
			free(ri[i]->priors);
		}
		
		ri[i]->RNAME = 0;
		ri[i]->POS = 0;
		ri[i]->MRNM = 0;
		ri[i]->MPOS = 0;
		ri[i]->ISIZE = 0;
		ri[i]->remain = 0;
		//free(ri[i]->strand);
		ri[i]->priors = 0;
		ri[i]->seq = 0;
		ri[i]->name = 0;
		ri[i]->qual = 0;
		ri[i]->len = 0;
		ri[i]->md = 0;
		ri[i]->cigar = 0;
		ri[i]->a = 0;
		//ri[i]->strand = 0;
	}
	///fprintf(stderr,"READING....\n");
	
	while(fgets(line, MAX_LINE, file)){
		//reading name .... 
		//fprintf(stderr,"%s",line);
		if(line[0] != '#'){ 
			if((line[0] == '@' && !set)|| (line[0] == '>' && !set)){
				//set sequence length of previous read
				//if(park_pos != -1){
				//	fprintf(stderr,"%s	%s	%s\n",ri[park_pos]->name,ri[park_pos]->seq);
				//}
				
				//check if there is still space.... 
				//if(param->num_query == size){
				//	fseek (file , -  strlen(line) , SEEK_CUR);
					
				//	return size;
				//}
				park_pos++;
				len = 0;
				seq_p = 1;
				for(i = 1;i < MAX_LINE;i++){
					len++;
					if(isspace((int)line[i])){
						break;
					}
					
				}
				
				ri[park_pos]->hits[0] = 0;
				ri[park_pos]->flag[0] = 0;
				ri[park_pos]->name = malloc(sizeof(unsigned char)* (len+1));
				for(i = 1;i < MAX_LINE;i++){
					
					if(isspace((int)line[i])){
						ri[park_pos]->name[i-1] = 0;
						break;
					}
					ri[park_pos]->name[i-1] = line[i];
				}
				//fprintf(stderr,"LEN:%d	%s\n",len,ri[park_pos]->name);
				
				set = 1;
				size++;
				//get ready to read quality if present  
			}else if(line[0] == '+' && !set){
				seq_p = 0;
				set = 1;
				fastq = 1;
				//reading sequence or quality  
			}else{	
				if(set){
					if(seq_p){
						len = 0;
						for(i = 0;i < MAX_LINE;i++){
							len++;
							if(isspace((int)line[i])){
								break;
							}
							
						}
						//fprintf(stderr,"SEQ LEN:%d	%s\n",len,line);
						ri[park_pos]->seq = malloc(sizeof(unsigned char)* (len+1));
						for(i = 0;i < MAX_LINE;i++){
							
							if(isspace((int)line[i])){
								ri[park_pos]->seq[i] = 0;
								break;
							}
							ri[park_pos]->seq[i] = nuc_code5[(int)line[i]];
						}
						ri[park_pos]->len = len-1;
						if(param->num_query == size+1 && fastq == 0){
							return size;
						}
					}else{
						len = 0;
						for(i = 0;i < MAX_LINE;i++){
							len++;
							if(isspace((int)line[i])){
								break;
							}
							
						}
						//fprintf(stderr,"QUAL LEN:%d\n",len);
						ri[park_pos]->qual = malloc(sizeof(unsigned char)* (len+1));
						for(i = 0;i < MAX_LINE;i++){
							
							if(isspace((int)line[i])){
								ri[park_pos]->qual[i] = 0;
								break;
							}
							ri[park_pos]->qual[i] = line[i];
						}
						
						if(param->num_query == size+1){
							return size;
						}
					}
				}
				set = 0;
			}
		}
	}
	return size;
}


void unaligned_to_sam(struct read_info* ri,FILE* out)
{
	int i,c;
	char nuc[] = "ACGTN";
	char pline[500];
	char pseq[500];
	for(i = 0;i < ri->len;i++){
		if(ri->seq[i] == 12){
			pseq[i] = 'N';
		}else{
			pseq[i] = nuc[(int)ri->seq[i] & 3 ];
		}
	}
	pseq[ri->len]  = 0;
	
	
	if(ri->qual){
		sprintf(pline,"%s\t%d\t%s\t%d\t%d\t%s\t%s\t%d\t%d\t%s\t%s",
				ri->name ,//1 QNAME Query NAME of the read or the read pair 
				4,//2 FLAG bitwise FLAG (pairing, strand, mate strand, etc.) 
				"*",//3 RNAME Reference sequence NAME 
				0,//4 POS 1-based leftmost POSition of clipped alignment 
				0,//5 MAPQ MAPping Quality (Phred-scaled) 
				"*",//6 CIGAR extended CIGAR string (operations: MIDNSHP) 
				"*",//7 MRNM Mate Reference NaMe (‘=’ if same as RNAME) 
				0,//8 MPOS 1-based leftmost Mate POSition 
				0,//9 ISIZE inferred Insert SIZE 
				pseq,//10 SEQ query SEQuence on the same strand as the reference 
				ri->qual//11 QUAL query QUALity (ASCII-33=Phred base quality) 
				);
	}else{
		sprintf(pline,"%s\t%d\t%s\t%d\t%d\t%s\t%s\t%d\t%d\t%s\t%s",
				ri->name ,//1 QNAME Query NAME of the read or the read pair 
				4,//2 FLAG bitwise FLAG (pairing, strand, mate strand, etc.) 
				"*",//3 RNAME Reference sequence NAME 
				0,//4 POS 1-based leftmost POSition of clipped alignment 
				0,//5 MAPQ MAPping Quality (Phred-scaled) 
				"*",//6 CIGAR extended CIGAR string (operations: MIDNSHP) 
				"*",//7 MRNM Mate Reference NaMe (‘=’ if same as RNAME) 
				0,//8 MPOS 1-based leftmost Mate POSition 
				0,//9 ISIZE inferred Insert SIZE 
				pseq,//10 SEQ query SEQuence on the same strand as the reference 
				"*"//11 QUAL query QUALity (ASCII-33=Phred base quality) 
				);
	}
	c = strlen(pline);
	pline[c] = 0;
	fprintf(out, "%s\n",pline);
}


void free_aln(struct fasta*aln)
{
	int i;
	for (i =0;i < aln->numseq;i++){
		free(aln->sn[i]);
	}
	free(aln->mer_hash);
	if(aln->suffix){
		free(aln->suffix);
	}
	
	//free(aln->mer_hash);
	free(aln->s_index);
	free(aln->string);
	free(aln->sn);
	
	free(aln);
}



