/*
 
 Contains code from Numerical recipes in C
 
*/

#include "nuc_code.h"
#include "interface.h"
#include "io.h"
#include "rRNAfilter.h"
#include "misc.h"



char* shorten_pathname(char* p)
{
	int i;
	char* tmp = p;
	for(i = 0; i< strlen(p);i++){
		if(p[i] == '/'){
			tmp = p+i +1;
		}
	}
	return tmp;
}

unsigned char* reverse_complement2(unsigned char* p,int len)
{
	unsigned char* tmp = malloc(sizeof(unsigned char)*MAX_LINE);
	int i,c;
	c = 0;
	for(i =len-1; i >= 0;i--){
		tmp[c] = rev_nuc_code5[(int)p[i]];
		c++;
	}
	tmp[c] = 0;
	for(i= 0; i < len;i++){
		p[i] = tmp[i];
	}
	free(tmp);
	return p;
}

unsigned char* reverse_without_complement(unsigned char* p,int len)
{
	unsigned char* tmp = malloc(sizeof(unsigned char)*MAX_LINE);
	int i,c;
	c = 0;
	for(i =len-1; i >= 0;i--){
		tmp[c] = (int)p[i];
		c++;
	}
	tmp[c] = 0;
	for(i= 0; i < len;i++){
		p[i] = tmp[i];
	}
	free(tmp);
	return p;
}


float betai(float a, float b, float x)
{

	//float betacf(float a, float b, float x);
	///float gammln(float xx);
	//void nrerror(char error_text[]);
	float bt;
	if (x < 0.0 || x > 1.0){
		fprintf(stderr,"Bad x in routine betai\n");
	}
	if (x == 0.0 || x == 1.0){
		bt = 0.0;
	}else{	//Factors in front of the continued fraction.
		bt = exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.0-x));
	}
	if (x < (a+1.0)/(a+b+2.0)){	//Use continued fraction directly.
		return bt*betacf(a,b,x)/a;
	}else{//	Use continued fraction after making the sym-
		return 1.0-bt*betacf(b,a,1.0-x)/b;//	metry transformation.
	}
	
}

float betacf(float a, float b, float x)
{

	int m,m2;
	float aa,c,d,del,h,qab,qam,qap;
	qab=a+b;
	qap=a+1.0;
	qam=a-1.0;
	c=1.0;
	d=1.0-qab*x/qap;
	if (fabs(d) < FPMIN){
		d=FPMIN; 
	}
	d=1.0/d;
	h=d;
	for (m=1;m<=MAXIT;m++) {
		m2=2*m;
		aa=m*(b-m)*x/((qam+m2)*(a+m2));
		d=1.0+aa*d;
		
		if (fabs(d) < FPMIN){
			d=FPMIN;
		}
		c=1.0+aa/c;
		if (fabs(c) < FPMIN){
			c=FPMIN;
		}
		d=1.0/d;
		h *= d*c;
		aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
		
		d=1.0+aa*d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=1.0+aa/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=d*c;
		h *= del;
		if (fabs(del-1.0) < EPS){
			break;
		}
		
	}
	if (m > MAXIT){
		//fprintf(stderr,"%f	%f	%f\n",a,b,x);
	//	fprintf(stderr,"a or b too big, or MAXIT too small in betacf\n");
	}
	return h;

}

double binomial_distribution(double p , int n, int k)
{
	double fac = gammln((double)n + 1.0);
	if(k < 0){
	//fprintf(stderr,"Bad k in binomialdist\n");
	}
	if(k > n){
		return 0;
	}
	return exp(k*log(p) +(n-k)*log(1.0 - p) + fac -gammln(k + 1.0) - gammln(n-k+1.0)); 
	
}



double gammln(const double xx)
{
	int j;
	double x,tmp,y,ser;
	static const double cof[14] = {57.1562356658629235,-59.5979603554754912,14.1360979747417471, -0.491913816097620199,0.339946499848118887e-4,0.465236289270485756e-4, -0.983744753048795646e-4, 0.0158088703224912494e-3,-0.210264441724104883e-3, 0.217439618115212643e-3, -0.164318106536763890e-3,0.888182239838527433e-4, -0.261908384015814087e-4, 0.368991826595316234e-5};
	if(xx <= 0.0){
		//fprintf(stderr,"bar arg in gammln");
	}
	y = xx;
	x = xx;
	tmp = x+5.24218750000000000;
	tmp = (x + 0.5) * log(tmp) - tmp;
	ser = 0.999999999999997092;
	for(j = 0; j < 14;j++){
		ser += cof[j] / ++y;
	}
	return tmp + log(2.5066282746310005*ser/x);
}



int byg_end(const char* pattern,const char*text)
{
	const char* tmp = 0;
	int Tc;
	int i  = 0;
	int s = 0;
	int T[256];
	for (i = 0;i < 256;i++){ 
		T[i] = 0; 
	}
	
	int m = (int)strlen(pattern);
	int n = (int)strlen(text);
	if (m > n){
		i = m;
		m = n;
		n = i;
		tmp = text;
		text = pattern;
		pattern = tmp;
	}
	
	int mb = (1 << (m-1));
	
	for (i= 0;i < m;i++){
		T[(int)pattern[i]] |= (1 << i);
	}
	
	for (i = 0;i < n;i++){
		s <<= 1;
		s |= 1;
		if(!text[i]){
			return -1;
		}
		Tc = T[(int)text[i]];
		s &= Tc;
		if(s & mb){
			return i+1;
		}
	}
	return 0;
}




int bpm(const unsigned char* t,const unsigned char* p,int n,int m,int limit)
{
	register unsigned long int i;//,c;
	unsigned long int diff;
	unsigned long int B[5];
	if(m > 31){
		m = 31;
	}
	
	unsigned long int k = m;
	//static int counter = 0;
	register unsigned long int VP,VN,D0,HN,HP,X;
	
	long int MASK = 0;
	//c = 0;
	
	diff = m;
	
	for(i = 0; i < 5;i++){
		B[i] = 0;
	}
	
	for(i = 0; i < m;i++){
		B[(int)(p[i] & 0x3)] |= (1ul << i);
	}
	
	//c = 0;
	VP = 0xFFFFFFFFFFFFFFFFul;
	VN = 0ul;
	m--;
	MASK = 1ul << m;
	for(i = 0; i < n;i++){
		X = (B[(int)(t[i] &0x3)  ] | VN);
		D0 = ((VP+(X&VP)) ^ VP) | X ;
		HN = VP & D0;
		HP = VN | ~(VP | D0);
		X = HP << 1ul;
		VN = X & D0;
		VP = (HN << 1ul) | ~(X | D0);
		diff += (HP & MASK) >> m;
		diff -= (HN & MASK) >> m;
		if(diff < k){
			k = diff;
			//fprintf(stderr,"%ld	%ld\n",i,k);
			//if(k <= limit){
			//	return (int)k;
			//}
			
		}
	}
	return (int)k;
}




int validate_bpm_sse(unsigned char**  query, int* query_lengths,unsigned char* t,int n,int num)
{
	int i,j;
	int len = 0;
	//struct seq_info* si = 0;
	//unsigned long int new; 
	unsigned int _MM_ALIGN16 nuc[16];
	//int _MM_ALIGN16 positions[4];
	//int _MM_ALIGN16 errors[4];
	
	unsigned int _MM_ALIGN16 lengths[4];
	
	__m128i VP,VN,D0,HN,HP,X,MASK,K,NOTONE,POS,diff,zero,one;
	__m128i* nuc_p;
	__m128i xmm1,xmm2;
	//long int MASK = 0;
	
	for(i = 0; i < 16;i++){
		nuc[i] = 0ul;
	}
	
	for(i = 0; i < num;i++){
		//si = qs->seq_info[abs(assignment[i])];
		len = query_lengths[i];
		if(len > 31){
			len = 31;
		}
		
		lengths[i] = len;
		
		
		
		for(j = 0; j < len;j++){
			nuc[((int)(query[i][j] & 0x3u) << 2) + i] |=  (1ul << j);// (unsigned long int)(len-1-j));
			//			fprintf(stderr,"%c",si->seq[j]+65);
		}
		//}else{
		//	for(j = 0; j < len;j++){
		//		nuc[((int)(si->reverse_seq[j] & 0x3u) << 2) + i] |=  (1ul << (unsigned long int)(len-1-j));
		//		//			fprintf(stderr,"%c",si->reverse_seq[j] + 65);
		//	}
		//}
		//	fprintf(stderr,"\n" );
	}
	nuc_p = (__m128i*) nuc;
	//S = _mm_set1_epi32(0);
	zero = _mm_set1_epi32(0);
	one = _mm_set1_epi32(1);
	diff =  _mm_load_si128 ( (__m128i*) lengths );  // _mm_set1_epi32(m);
	VP =  _mm_set1_epi32(0xFFFFFFFFu);
	VN =  _mm_set1_epi32(0); 
	NOTONE =  _mm_set1_epi32(0xFFFFFFFF);
	K =  _mm_set1_epi32(0x7FFFFFFF);
	POS = _mm_set1_epi32(0);
	
	//VP = 0xFFFFFFFFFFFFFFFFul;
	//VN = 0ul;
	
	for(i = 0; i< 4;i++){
		lengths[i]--;
		lengths[i] = 1 << lengths[i];
		//	fprintf(stderr,"%d	%d	LEN:%d\n",i,num,lengths[i]);
	}
	
	// m--;
	MASK =  _mm_load_si128 ( (__m128i*) lengths ); //  _mm_set1_epi32(1ul << m);
	for(i = 0; i < n ;i++){
		//fprintf(stderr,"%c",*t + 65);
		X = _mm_or_si128 (*(nuc_p +( (int)(*t)  & 0x3u) ) , VN);
		//X = (B[(int) *t] | VN);
		xmm1 = _mm_and_si128(X, VP);
		xmm2 = _mm_add_epi32(VP ,xmm1);
		xmm1 = _mm_xor_si128 (xmm2, VP);
		D0 = _mm_or_si128(xmm1, X);
		//D0 = ((VP+(X&VP)) ^ VP) | X ;
		HN = _mm_and_si128(VP, D0);
		//HN = VP & D0;
		xmm1 = _mm_or_si128(VP, D0);
		xmm2 = _mm_andnot_si128 (xmm1,NOTONE);
		HP = _mm_or_si128(VN, xmm2);
		//HP = VN | ~(VP | D0);
		X = _mm_slli_epi32(HP,1);
		//X = HP << 1ul;
		VN = _mm_and_si128(X, D0);
		//VN = X & D0;
		xmm1 = _mm_slli_epi32(HN,1);
		xmm2 = _mm_or_si128(X, D0);
		xmm2 = _mm_andnot_si128 (xmm2,NOTONE);
		VP = _mm_or_si128(xmm1, xmm2);
		//VP = (HN << 1ul) | ~(X | D0);
		xmm1 = _mm_and_si128(HP, MASK);
		xmm2 = _mm_cmpgt_epi32(xmm1, zero);
		diff = _mm_add_epi32(diff , _mm_and_si128( xmm2, one));
		//diff += (HP & MASK) >> m;
		xmm1 = _mm_and_si128(HN, MASK);
		xmm2 = _mm_cmpgt_epi32(xmm1, zero);
		diff = _mm_sub_epi32(diff,  _mm_and_si128( xmm2, one));
		//diff -= (HN & MASK) >> m;
		xmm1 = _mm_cmplt_epi32(diff, K);
		xmm2 = _mm_and_si128(xmm1, diff);
		K = _mm_or_si128(xmm2, _mm_andnot_si128  (xmm1,K));
		//xmm2 = _mm_and_si128(xmm1, _mm_set1_epi32(i));
		//POS = _mm_or_si128(xmm2, _mm_andnot_si128  (xmm1,POS));
		t++;
	}
	//fprintf(stderr,"\n");
	//_mm_store_si128 ((__m128i*) positions, POS);
	//fprintf(stderr,"%d	%d	%d	%d	",positions[0],positions[1],positions[2],positions[3]);
	_mm_store_si128 ((__m128i*) query_lengths, K);
	//fprintf(stderr,"%d	%d	%d	%d\n",query_lengths[0],query_lengths[1],query_lengths[2],query_lengths[3]);
	
	/*for(i = 0; i < num;i++){
		si = qs->seq_info[abs(assignment[i])];
		//len = si->len;
		//lengths[i] = len;
		//	fprintf(stderr,"%d	%d	%d	%d	%d\n",  assignment[i] , si->len,num,out2[i],out1[i] + offset);
		if(assignment[i] >= 0){
			new = ((unsigned long int)(si->len - errors[i])) << 56ul;
			new |= ((unsigned long int) ((unsigned long int)offset - (unsigned long int) positions[i]) << 1ul);
			BinaryInsertionSortOne(si->match_units,LIST_STORE_SIZE ,new);
		}else{
			new = ((unsigned long int)(si->len - errors[i])) << 56ul;
			new |= ((unsigned long int) ((unsigned long int) offset - (unsigned long int) positions[i]) << 1ul);
			new |= 1ul ;
			BinaryInsertionSortOne(si->match_units,LIST_STORE_SIZE ,new);
		}
		
	}*/
	return 1;
}








